unit AcColorGrid;
{
      TAcColorGrid  TAcColorLine  
     AutoCAD Color Index (ACI). TAcColorGrid   
      24   10 , TAcColorLine 
       15- .

  :
    TAcColorGrid <- TCustomAcColorGrid <- TGraphicControl;
    TAcColorLine <- TCustomAcColorGrid <- TGraphicControl;
    
    Colors  TAcColorGridColors   
   :
    Colors.CurrentColor -     (    )
    Colors.CurrentTrueColor -       TColor
    Colors.BoundaryColor -   (..  /  )
    Colors.SelectedColor -    (    )

  :  .., 2006.       
  ,      .
}
interface

uses
  SysUtils, Classes, Controls, Types, Windows, Graphics, Messages, ActiveX;

type
  TAcColorsArray =  array[0..23, 0..9] of Integer;

  TAcColorsLine =  array[0..14] of Integer;

const
  AcColorsLine: TAcColorsLine = (1, 2, 3, 4, 5, 6, 7, 8, 9, 250, 251, 252, 253, 254, 255);

type
  TCustomAcColorGrid = class;
  
  TAcColorGridColors = class(TPersistent)
  private
    FBoundaryColor: TColor;
    FSelectedColor: Byte;
    FCurrentColor: Byte;
    FCurrentTrueColor: TColor;
    FOwner: TCustomAcColorGrid;
  protected
    function  GetOwner: TPersistent; override;
    procedure SetBoundaryColor(const Value: TColor); virtual;
    procedure SetSelectedColor(const Value: Byte); virtual;
  public
    constructor Create(AOwner: TCustomAcColorGrid);
    procedure Assign(Source: TPersistent); override;
    property CurrentColor: Byte read FCurrentColor write FCurrentColor;
    property CurrentTrueColor: TColor read FCurrentTrueColor write FCurrentTrueColor;
  published
    property BoundaryColor: TColor read FBoundaryColor write SetBoundaryColor default clBtnFace;
    property SelectedColor: Byte read FSelectedColor write SetSelectedColor default 10;
  end;

  TCustomAcColorGrid = class(TGraphicControl)
  private
    FColCount: Byte;
    FRowCount: Byte;
    FCellSize: Word;
    FCell: TPoint;
    FSelCell: TPoint;
    FColors: TAcColorGridColors;
    FOnMouseLeave: TNotifyEvent;
    FOnMouseEnter: TNotifyEvent;
    procedure SetColors(const Value: TAcColorGridColors);
    procedure CMMouseEnter(var Message: TMessage); message CM_MOUSEENTER;
    procedure CMMouseLeave(var Message: TMessage); message CM_MOUSELEAVE;
  protected
    function CalcCoordFromPoint(X, Y: Integer): TPoint; virtual; abstract;
    function CellRect(ACol, ARow: Longint): TRect; virtual;
    procedure PaintCell(ACol, ARow: Longint); virtual; abstract;
    procedure InitialColorsArray; virtual; abstract;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    property Cell: TPoint read FCell;
    property Colors: TAcColorGridColors read FColors write SetColors;
  published
    property OnMouseEnter: TNotifyEvent read FOnMouseEnter write FOnMouseEnter;
    property OnMouseLeave: TNotifyEvent read FOnMouseLeave write FOnMouseLeave;
    property Align;
    property Anchors;
    property DragCursor;
    property DragKind;
    property DragMode;
    property Enabled;
    property Constraints;
    property ParentShowHint;
    property ShowHint;
    property Visible;
    property OnContextPopup;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDock;
    property OnEndDrag;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnStartDock;
    property OnStartDrag;
  end;

  TAcColorGrid = class(TCustomAcColorGrid)
  private
    FAcColorsArray: TAcColorsArray;
  protected
    procedure Paint; override;
    procedure MouseDown(Button: TMouseButton; Shift: TShiftState;
      X, Y: Integer); override;
    procedure MouseMove(Shift: TShiftState; X, Y: Integer); override;
    function CalcCoordFromPoint(X, Y: Integer): TPoint; override;
    procedure PaintCell(ACol, ARow: Longint);override;
    procedure InitialColorsArray; override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    property Cell;
    property Colors;
  end;

  TAcColorLine = class(TCustomAcColorGrid)
  private
    FAcColorsLine: TAcColorsLine;
  protected
    procedure Paint; override;
    procedure MouseDown(Button: TMouseButton; Shift: TShiftState;
      X, Y: Integer); override;
    procedure MouseMove(Shift: TShiftState; X, Y: Integer); override;
    function CalcCoordFromPoint(X, Y: Integer): TPoint; override;
    procedure PaintCell(ACol, ARow: Longint);override;
    procedure InitialColorsArray; override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    property Cell;
    property Colors;
  end;

type
  AcColor = TOleEnum;
  
  TAcadColorRec = packed record
    I: Byte;
    R: Byte;
    G: Byte;
    B: Byte;
  end;

const
  AcadColorIndex: array[0..255] of TAcadColorRec = (
    (I:  0; R:255; G:  0; B:  0), (I:  1; R:255; G:  0; B:  0),
    (I:  2; R:255; G:255; B:  0), (I:  3; R:  0; G:255; B:  0),
    (I:  4; R:  0; G:255; B:255), (I:  5; R:  0; G:  0; B:255),
    (I:  6; R:255; G:  0; B:255), (I:  7; R:255; G:255; B:255),
    (I:  8; R:128; G:128; B:128), (I:  9; R:192; G:192; B:192),
    (I: 10; R:255; G:  0; B:  0), (I: 11; R:255; G:127; B:127),
    (I: 12; R:204; G:  0; B:  0), (I: 13; R:204; G:102; B:102),
    (I: 14; R:153; G:  0; B:  0), (I: 15; R:153; G: 76; B: 76),
    (I: 16; R:127; G:  0; B:  0), (I: 17; R:127; G: 63; B: 63),
    (I: 18; R: 76; G:  0; B:  0), (I: 19; R: 76; G: 38; B: 38),
    (I: 20; R:255; G: 63; B:  0), (I: 21; R:255; G:159; B:127),
    (I: 22; R:204; G: 51; B:  0), (I: 23; R:204; G:127; B:102),
    (I: 24; R:153; G: 38; B:  0), (I: 25; R:153; G: 95; B: 76),
    (I: 26; R:127; G: 31; B:  0), (I: 27; R:127; G: 79; B: 63),
    (I: 28; R: 76; G: 19; B:  0), (I: 29; R: 76; G: 47; B: 38),
    (I: 30; R:255; G:127; B:  0), (I: 31; R:255; G:191; B:127),
    (I: 32; R:204; G:102; B:  0), (I: 33; R:204; G:153; B:102),
    (I: 34; R:153; G: 76; B:  0), (I: 35; R:153; G:114; B: 76),
    (I: 36; R:127; G: 63; B:  0), (I: 37; R:127; G: 95; B: 63),
    (I: 38; R: 76; G: 38; B:  0), (I: 39; R: 76; G: 57; B: 38),
    (I: 40; R:255; G:191; B:  0), (I: 41; R:255; G:223; B:127),
    (I: 42; R:204; G:153; B:  0), (I: 43; R:204; G:178; B:102),
    (I: 44; R:153; G:114; B:  0), (I: 45; R:153; G:133; B: 76),
    (I: 46; R:127; G: 95; B:  0), (I: 47; R:127; G:111; B: 63),
    (I: 48; R: 76; G: 57; B:  0), (I: 49; R: 76; G: 66; B: 38),
    (I: 50; R:255; G:255; B:  0), (I: 51; R:255; G:255; B:127),
    (I: 52; R:204; G:204; B:  0), (I: 53; R:204; G:204; B:102),
    (I: 54; R:153; G:153; B:  0), (I: 55; R:153; G:153; B: 76),
    (I: 56; R:127; G:127; B:  0), (I: 57; R:127; G:127; B: 63),
    (I: 58; R: 76; G: 76; B:  0), (I: 59; R: 76; G: 76; B: 38),
    (I: 60; R:191; G:255; B:  0), (I: 61; R:223; G:255; B:127),
    (I: 62; R:153; G:204; B:  0), (I: 63; R:178; G:204; B:102),
    (I: 64; R:114; G:153; B:  0), (I: 65; R:133; G:153; B: 76),
    (I: 66; R: 95; G:127; B:  0), (I: 67; R:111; G:127; B: 63),
    (I: 68; R: 57; G: 76; B:  0), (I: 69; R: 66; G: 76; B: 38),
    (I: 70; R:127; G:255; B:  0), (I: 71; R:191; G:255; B:127),
    (I: 72; R:102; G:204; B:  0), (I: 73; R:153; G:204; B:102),
    (I: 74; R: 76; G:153; B:  0), (I: 75; R:114; G:153; B: 76),
    (I: 76; R: 63; G:127; B:  0), (I: 77; R: 95; G:127; B: 63),
    (I: 78; R: 38; G: 76; B:  0), (I: 79; R: 57; G: 76; B: 38),
    (I: 80; R: 63; G:255; B:  0), (I: 81; R:159; G:255; B:127),
    (I: 82; R: 51; G:204; B:  0), (I: 83; R:127; G:204; B:102),
    (I: 84; R: 38; G:153; B:  0), (I: 85; R: 95; G:153; B: 76),
    (I: 86; R: 31; G:127; B:  0), (I: 87; R: 79; G:127; B: 63),
    (I: 88; R: 19; G: 76; B:  0), (I: 89; R: 47; G: 76; B: 38),
    (I: 90; R:  0; G:255; B:  0), (I: 91; R:127; G:255; B:127),
    (I: 92; R:  0; G:204; B:  0), (I: 93; R:102; G:204; B:102),
    (I: 94; R:  0; G:153; B:  0), (I: 95; R: 76; G:153; B: 76),
    (I: 96; R:  0; G:127; B:  0), (I: 97; R: 63; G:127; B: 63),
    (I: 98; R:  0; G: 76; B:  0), (I: 99; R: 38; G: 76; B: 38),
    (I:100; R:  0; G:255; B: 63), (I:101; R:127; G:255; B:159),
    (I:102; R:  0; G:204; B: 51), (I:103; R:102; G:204; B:127),
    (I:104; R:  0; G:153; B: 38), (I:105; R: 76; G:153; B: 95),
    (I:106; R:  0; G:127; B: 31), (I:107; R: 63; G:127; B: 79),
    (I:108; R:  0; G: 76; B: 19), (I:109; R: 38; G: 76; B: 47),
    (I:110; R:  0; G:255; B:127), (I:111; R:127; G:255; B:191),
    (I:112; R:  0; G:204; B:102), (I:113; R:102; G:204; B:153),
    (I:114; R:  0; G:153; B: 76), (I:115; R: 76; G:153; B:114),
    (I:116; R:  0; G:127; B: 63), (I:117; R: 63; G:127; B: 95),
    (I:118; R:  0; G: 76; B: 38), (I:119; R: 38; G: 76; B: 57),
    (I:120; R:  0; G:255; B:191), (I:121; R:127; G:255; B:223),
    (I:122; R:  0; G:204; B:153), (I:123; R:102; G:204; B:178),
    (I:124; R:  0; G:153; B:114), (I:125; R: 76; G:153; B:133),
    (I:126; R:  0; G:127; B: 95), (I:127; R: 63; G:127; B:111),
    (I:128; R:  0; G: 76; B: 57), (I:129; R: 38; G: 76; B: 66),
    (I:130; R:  0; G:255; B:255), (I:131; R:127; G:255; B:255),
    (I:132; R:  0; G:204; B:204), (I:133; R:102; G:204; B:204),
    (I:134; R:  0; G:153; B:153), (I:135; R: 76; G:153; B:153),
    (I:136; R:  0; G:127; B:127), (I:137; R: 63; G:127; B:127),
    (I:138; R:  0; G: 76; B: 76), (I:139; R: 38; G: 76; B: 76),
    (I:140; R:  0; G:191; B:255), (I:141; R:127; G:223; B:255),
    (I:142; R:  0; G:153; B:204), (I:143; R:102; G:178; B:204),
    (I:144; R:  0; G:114; B:153), (I:145; R: 76; G:133; B:153),
    (I:146; R:  0; G: 95; B:127), (I:147; R: 63; G:111; B:127),
    (I:148; R:  0; G: 57; B: 76), (I:149; R: 38; G: 66; B: 76),
    (I:150; R:  0; G:127; B:255), (I:151; R:127; G:191; B:255),
    (I:152; R:  0; G:102; B:204), (I:153; R:102; G:153; B:204),
    (I:154; R:  0; G: 76; B:153), (I:155; R: 76; G:114; B:153),
    (I:156; R:  0; G: 63; B:127), (I:157; R: 63; G: 95; B:127),
    (I:158; R:  0; G: 38; B: 76), (I:159; R: 38; G: 57; B: 76),
    (I:160; R:  0; G: 63; B:255), (I:161; R:127; G:159; B:255),
    (I:162; R:  0; G: 51; B:204), (I:163; R:102; G:127; B:204),
    (I:164; R:  0; G: 38; B:153), (I:165; R: 76; G: 95; B:153),
    (I:166; R:  0; G: 31; B:127), (I:167; R: 63; G: 79; B:127),
    (I:168; R:  0; G: 19; B: 76), (I:169; R: 38; G: 47; B: 76),
    (I:170; R:  0; G:  0; B:255), (I:171; R:127; G:127; B:255),
    (I:172; R:  0; G:  0; B:204), (I:173; R:102; G:102; B:204),
    (I:174; R:  0; G:  0; B:153), (I:175; R: 76; G: 76; B:153),
    (I:176; R:  0; G:  0; B:127), (I:177; R: 63; G: 63; B:127),
    (I:178; R:  0; G:  0; B: 76), (I:179; R: 38; G: 38; B: 76),
    (I:180; R: 63; G:  0; B:255), (I:181; R:159; G:127; B:255),
    (I:182; R: 51; G:  0; B:204), (I:183; R:127; G:102; B:204),
    (I:184; R: 38; G:  0; B:153), (I:185; R: 95; G: 76; B:153),
    (I:186; R: 31; G:  0; B:127), (I:187; R: 79; G: 63; B:127),
    (I:188; R: 19; G:  0; B: 76), (I:189; R: 47; G: 38; B: 76),
    (I:190; R:127; G:  0; B:255), (I:191; R:191; G:127; B:255),
    (I:192; R:102; G:  0; B:204), (I:193; R:153; G:102; B:204),
    (I:194; R: 76; G:  0; B:153), (I:195; R:114; G: 76; B:153),
    (I:196; R: 63; G:  0; B:127), (I:197; R: 95; G: 63; B:127),
    (I:198; R: 38; G:  0; B: 76), (I:199; R: 57; G: 38; B: 76),
    (I:200; R:191; G:  0; B:255), (I:201; R:223; G:127; B:255),
    (I:202; R:153; G:  0; B:204), (I:203; R:178; G:102; B:204),
    (I:204; R:114; G:  0; B:153), (I:205; R:133; G: 76; B:153),
    (I:206; R: 95; G:  0; B:127), (I:207; R:111; G: 63; B:127),
    (I:208; R: 57; G:  0; B: 76), (I:209; R: 66; G: 38; B: 76),
    (I:210; R:255; G:  0; B:255), (I:211; R:255; G:127; B:255),
    (I:212; R:204; G:  0; B:204), (I:213; R:204; G:102; B:204),
    (I:214; R:153; G:  0; B:153), (I:215; R:153; G: 76; B:153),
    (I:216; R:127; G:  0; B:127), (I:217; R:127; G: 63; B:127),
    (I:218; R: 76; G:  0; B: 76), (I:219; R: 76; G: 38; B: 76),
    (I:220; R:255; G:  0; B:191), (I:221; R:255; G:127; B:223),
    (I:222; R:204; G:  0; B:153), (I:223; R:204; G:102; B:178),
    (I:224; R:153; G:  0; B:114), (I:225; R:153; G: 76; B:133),
    (I:226; R:127; G:  0; B: 95), (I:227; R:127; G: 63; B:111),
    (I:228; R: 76; G:  0; B: 57), (I:229; R: 76; G: 38; B: 66),
    (I:230; R:255; G:  0; B:127), (I:231; R:255; G:127; B:191),
    (I:232; R:204; G:  0; B:102), (I:233; R:204; G:102; B:153),
    (I:234; R:153; G:  0; B: 76), (I:235; R:153; G: 76; B:114),
    (I:236; R:127; G:  0; B: 63), (I:237; R:127; G: 63; B: 95),
    (I:238; R: 76; G:  0; B: 38), (I:239; R: 76; G: 38; B: 57),
    (I:240; R:255; G:  0; B: 63), (I:241; R:255; G:127; B:159),
    (I:242; R:204; G:  0; B: 51), (I:243; R:204; G:102; B:127),
    (I:244; R:153; G:  0; B: 38), (I:245; R:153; G: 76; B: 95),
    (I:246; R:127; G:  0; B: 31), (I:247; R:127; G: 63; B: 79),
    (I:248; R: 76; G:  0; B: 19), (I:249; R: 76; G: 38; B: 47),
    (I:250; R: 51; G: 51; B: 51), (I:251; R: 91; G: 91; B: 91),
    (I:252; R:132; G:132; B:132), (I:253; R:173; G:173; B:173),
    (I:254; R:214; G:214; B:214), (I:255; R:255; G:255; B:255));

function AcadColorToColor(AColor: AcColor): TColor;

procedure Register;

implementation

uses
  Math;

procedure Register;
begin
  RegisterComponents('AutoCAD', [TAcColorGrid, TAcColorLine]);
end;

function AcadColorToColor(AColor: AcColor): TColor;
begin
  if AColor in [(Low(AcadColorIndex) + 1)..High(AcadColorIndex)] then
    Result:= RGB(AcadColorIndex[AColor].R, AcadColorIndex[AColor].G, AcadColorIndex[AColor].B)
  else
    Result:= clWhite;
end;

{ TAcColorGridColors }

procedure TAcColorGridColors.Assign(Source: TPersistent);
begin
  if Source is TAcColorGridColors then
  begin
    FBoundaryColor:= (Source as TAcColorGridColors).BoundaryColor;
    FSelectedColor:= (Source as TAcColorGridColors).SelectedColor;
    FCurrentColor:= (Source as TAcColorGridColors).CurrentColor;
    FCurrentTrueColor:= (Source as TAcColorGridColors).CurrentTrueColor;
    FOwner.Assign((Source as TAcColorGridColors).FOwner);
  end else inherited;
end;

constructor TAcColorGridColors.Create(AOwner: TCustomAcColorGrid);
begin
  FOwner:= AOwner;
  FBoundaryColor:= clBtnFace;
  SelectedColor:= 10;
end;

function TAcColorGridColors.GetOwner: TPersistent;
begin
  Result:= FOwner;
end;

procedure TAcColorGridColors.SetBoundaryColor(const Value: TColor);
begin
  if FBoundaryColor <> Value then
  begin
    FBoundaryColor:= Value;
    FOwner.Invalidate;
  end;
end;

procedure TAcColorGridColors.SetSelectedColor(const Value: Byte);
var
  c, r: Integer;
  Found: Boolean;
  CellCoord: TPoint;
begin
  if FSelectedColor <> Value then
  begin
    FSelectedColor:= Value;
    FOwner.FSelCell:= Point(-1, -1);
    Found:= False;
    if FOwner is TAcColorGrid then
    begin
      for r:= 0 to 9 do
        for c:= 0 to 23 do
          if TAcColorGrid(FOwner).FAcColorsArray[c, r] = FSelectedColor then
          begin
            Found:= True;
            CellCoord.X:= c;
            CellCoord.Y:= r;
            Break;
          end;
    end else
    if FOwner is TAcColorLine then
    begin
      for c:= 0 to 14 do
        if TAcColorLine(FOwner).FAcColorsLine[c] = FSelectedColor then
        begin
          Found:= True;
          CellCoord.X:= c;
          CellCoord.Y:= 0;
          Break;
        end;
    end;
    if Found then
    begin
      FOwner.FSelCell:= CellCoord;

    end;
    FOwner.Invalidate;
  end;
end;

{ TCustomAcColorGrid }

function TCustomAcColorGrid.CellRect(ACol, ARow: Integer): TRect;
begin
  Result:= Rect(ACol * FCellSize, ARow * FCellSize,
               (ACol + 1) * FCellSize - 1, ARow * FCellSize + FCellSize - 1);
end;

procedure TCustomAcColorGrid.CMMouseEnter(var Message: TMessage);
begin
  inherited;
  if Assigned(FOnMouseEnter) then FOnMouseEnter(Self);
end;

procedure TCustomAcColorGrid.CMMouseLeave(var Message: TMessage);
begin
  inherited;
  if Assigned(FOnMouseLeave) then FOnMouseLeave(Self);
end;

constructor TCustomAcColorGrid.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  Width:= 360;
  Height:= 150;
  FCell:= Point(-1, -1);
  FSelCell:= Point(-1, -1);
  InitialColorsArray;
  FColors:= TAcColorGridColors.Create(Self);
end;

destructor TCustomAcColorGrid.Destroy;
begin
  FColors.Free;
  inherited Destroy;
end;

procedure TCustomAcColorGrid.SetColors(const Value: TAcColorGridColors);
begin
  FColors.Assign(Value);
end;

{ TAcColorGrid }

function TAcColorGrid.CalcCoordFromPoint(X, Y: Integer): TPoint;
var
  i, N: Integer;
begin
  Result:= Point(-1, -1);
  N:= 0;
  for i:= 0 to FColCount - 1 do
  begin
    Inc(N, FCellSize);
    if X < N then
    begin
      Result.X:= i;
      Break;
    end;
  end;
  N:= 0;
  for i:= 0 to FRowCount - 1 do
  begin
    Inc(N, FCellSize);
    if Y < N then
    begin
      Result.Y:= i;
      Break;
    end;
  end;
end;

constructor TAcColorGrid.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FColCount:= 24;
  FRowCount:= 10;
  FCellSize:= Min(Width div FColCount, Height div FRowCount);
end;

destructor TAcColorGrid.Destroy;
begin
  inherited Destroy;
end;

procedure TAcColorGrid.InitialColorsArray;
var
  c, r: Integer;
begin
  for r:= 0 to 9 do
    for c:= 0 to 23 do
      begin
        if (r >= 0) and (r <= 4) then
          FAcColorsArray[c, r]:= -2 * r + 18 + c * 10;
        if (r > 4) and (r <= 9) then
          FAcColorsArray[c, r]:= 2 * r + 1 + c * 10;
      end;
end;

procedure TAcColorGrid.MouseDown(Button: TMouseButton; Shift: TShiftState;
  X, Y: Integer);
var
  OldCell: TPoint;
begin
  OldCell:= FSelCell;
  FSelCell:= CalcCoordFromPoint(X, Y);
  FColors.SelectedColor:= FAcColorsArray[FSelCell.X, FSelCell.Y];
  inherited MouseDown(Button, Shift, X, Y);
  PaintCell(OldCell.X, OldCell.Y);
  PaintCell(FSelCell.X, FSelCell.Y);
end;

procedure TAcColorGrid.MouseMove(Shift: TShiftState; X, Y: Integer);
var
  OldCell: TPoint;
begin
  OldCell:= FCell;
  FCell:= CalcCoordFromPoint(X, Y);
  FColors.CurrentColor:= FAcColorsArray[FCell.X, FCell.Y];
  FColors.CurrentTrueColor:= AcadColorToColor(FColors.CurrentColor);
  inherited MouseMove(Shift, X, Y);
  PaintCell(OldCell.X, OldCell.Y);
  PaintCell(FCell.X, FCell.Y);
end;

procedure TAcColorGrid.Paint;
var
  Col, Row: Integer;
begin
  Canvas.Brush.Color:= FColors.BoundaryColor;
  Canvas.FillRect(ClientRect);
  for Col:= 0 to FColCount - 1 do
  begin
    for Row:= 0 to FRowCount - 1 do
    begin
      PaintCell(Col, Row);
    end;
  end;
end;

procedure TAcColorGrid.PaintCell(ACol, ARow: Integer);
var
  R: TRect;
begin
  if (ACol > FColCount - 1) or (ARow > FRowCount - 1) or
     (ACol < 0) or (ARow < 0) then Exit;
  R:= CellRect(ACol, ARow);
  Canvas.Brush.Color:= AcadColorToColor(FAcColorsArray[ACol, ARow]);
  Canvas.FillRect(R);
  if (ACol = FCell.X) and (ARow = FCell.Y) then
  begin
    DrawEdge(Canvas.Handle, R, EDGE_RAISED, BF_RECT);
  end else
  if (ACol = FSelCell.X) and (ARow = FSelCell.Y) then
  begin
    DrawEdge(Canvas.Handle, R, EDGE_ETCHED, BF_RECT);
  end;
end;

{ TAcColorLine }

function TAcColorLine.CalcCoordFromPoint(X, Y: Integer): TPoint;
var
  i, N: Integer;
begin
  Result:= Point(-1, 0);
  N:= 0;
  for i:= 0 to FColCount - 1 do
  begin
    Inc(N, FCellSize);
    if X < N then
    begin
      Result.X:= i;
      Break;
    end;
  end;
end;

constructor TAcColorLine.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  Height:= 24;
  FColCount:= 15;
  FRowCount:= 1;
  FCellSize:= Min(Width div FColCount, Height div FRowCount);
end;

destructor TAcColorLine.Destroy;
begin
  inherited Destroy;
end;

procedure TAcColorLine.InitialColorsArray;
begin
  FAcColorsLine:= AcColorsLine;
end;

procedure TAcColorLine.MouseDown(Button: TMouseButton; Shift: TShiftState;
  X, Y: Integer);
var
  OldCell: TPoint;
begin
  OldCell:= FSelCell;
  FSelCell:= CalcCoordFromPoint(X, Y);
  FColors.SelectedColor:= FAcColorsLine[FSelCell.X];
  inherited MouseDown(Button, Shift, X, Y);
  PaintCell(OldCell.X, OldCell.Y);
  PaintCell(FSelCell.X, FSelCell.Y);
end;

procedure TAcColorLine.MouseMove(Shift: TShiftState; X, Y: Integer);
var
  OldCell: TPoint;
begin
  OldCell:= FCell;
  FCell:= CalcCoordFromPoint(X, Y);
  FColors.CurrentColor:= FAcColorsLine[FCell.X];
  FColors.CurrentTrueColor:= AcadColorToColor(FColors.CurrentColor);
  inherited MouseMove(Shift, X, Y);
  PaintCell(OldCell.X, OldCell.Y);
  PaintCell(FCell.X, FCell.Y);
end;

procedure TAcColorLine.Paint;
var
  Col: Integer;
begin
  Canvas.Brush.Color:= FColors.BoundaryColor;
  Canvas.FillRect(ClientRect);
  for Col:= 0 to FColCount - 1 do
  begin
    PaintCell(Col, 0);
  end;
end;

procedure TAcColorLine.PaintCell(ACol, ARow: Integer);
var
  R: TRect;
begin
  if (ACol > FColCount - 1) or (ACol < 0) then Exit;
  R:= CellRect(ACol, ARow);
  Canvas.Brush.Color:= AcadColorToColor(FAcColorsLine[ACol]);
  Canvas.FillRect(R);
  if (ACol = FCell.X) and (ARow = FCell.Y) then
  begin
    DrawEdge(Canvas.Handle, R, EDGE_RAISED, BF_RECT);
  end else
  if (ACol = FSelCell.X) and (ARow = FSelCell.Y) then
  begin
    DrawEdge(Canvas.Handle, R, EDGE_ETCHED, BF_RECT);
  end;
end;

initialization
  Classes.RegisterClass(TAcColorGridColors);

end.
