unit Unit1;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ComCtrls, ExtCtrls;

type
  TForm1 = class(TForm)
    btnConnect: TButton;
    GroupBox1: TGroupBox;
    Label3: TLabel;
    cbColors: TComboBox;
    chbShowColorValues: TCheckBox;
    btnAddLine: TButton;
    GroupBox2: TGroupBox;
    trbarR: TTrackBar;
    Label1: TLabel;
    trbarG: TTrackBar;
    Label2: TLabel;
    trbarB: TTrackBar;
    Label4: TLabel;
    Panel1: TPanel;
    Label5: TLabel;
    pbSample: TPaintBox;
    lblR: TLabel;
    lblG: TLabel;
    lblB: TLabel;
    btnAddTrueColorLine: TButton;
    chbUseGetIntfObj: TCheckBox;
    procedure FormCreate(Sender: TObject);
    procedure cbColorsDrawItem(Control: TWinControl; Index: Integer;
      Rect: TRect; State: TOwnerDrawState);
    procedure chbShowColorValuesClick(Sender: TObject);
    procedure btnConnectClick(Sender: TObject);
    procedure btnAddLineClick(Sender: TObject);
    procedure pbSamplePaint(Sender: TObject);
    procedure trbarRChange(Sender: TObject);
    procedure btnAddTrueColorLineClick(Sender: TObject);
  private
    { Private declarations }
    FShowColorValues: Boolean;
    procedure PopulateColorsList;
    procedure ColorTrackBarChange(Sender: TObject);
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

uses UAutoCAD;

{$R *.dfm}

procedure TForm1.PopulateColorsList;
var
  i: Integer;
begin
  with cbColors do
  begin
    Items.BeginUpdate;
    try
      Clear;
      for i:= 0 to 256 do
        Items.Add(IntToStr(i));
       ItemIndex:= 0; 
    finally
      Items.EndUpdate;
    end;
  end;
end;

procedure TForm1.FormCreate(Sender: TObject);
begin
  PopulateColorsList;
  cbColors.ItemIndex:= 10;
  Panel1.DoubleBuffered:= True;
  ColorTrackBarChange(trbarR);
  ColorTrackBarChange(trbarG);
  ColorTrackBarChange(trbarB);
end;

procedure TForm1.cbColorsDrawItem(Control: TWinControl; Index: Integer;
  Rect: TRect; State: TOwnerDrawState);
var
  R: TRect;
  C, AColor: TColor;
  P: TPoint;
  S: String;
begin
  AColor:= AcadColorToColor(Index);
  with cbColors.Canvas do
  begin
    //    
    C:= Brush.Color;
    Brush.Color:= cbColors.Color;
    FillRect(Rect);
    //   
    R:= Rect;
    R.Right:= R.Bottom - R.Top + R.Left;
    InflateRect(R, -2, -2);
    Brush.Color:= AColor;
    FillRect(R);
    //   
    Brush.Color:= clBlack;
    FrameRect(R);
    //   
    Brush.Color:= C;
    R:= Rect;
    R.Left:= R.Bottom - R.Top + R.Left;
    FillRect(R);
    //  
    R:= Rect;
    R.Left:= R.Bottom - R.Top + R.Left + 2;
    S:= cbColors.Items[Index];
    //      -    
    //    
    if FShowColorValues then S:= S + Format(' (%d, %d, %d)', [GetRValue(AColor),
                                                              GetGValue(AColor),
                                                              GetBValue(AColor)]);
    with R do
      P:= Point(Left, Top + (Bottom - Top - TextHeight(cbColors.Items[Index])) div 2);
    //      ""
    case Index of
        0: TextRect(R, P.X, P.Y, 'ByBlock');
      256: TextRect(R, P.X, P.Y, 'ByLayer');
    else
      TextRect(R, P.X, P.Y, S);
    end;
    //    , ...
    if odFocused in State then
    begin
      // ...    (   Rect)
      DrawFocusRect(Rect);
      // ...     
      DrawFocusRect(Classes.Rect(R.Left - 2, R.Top, R.Right, R.Bottom));
    end;
  end;
end;

procedure TForm1.chbShowColorValuesClick(Sender: TObject);
begin
  FShowColorValues:= (Sender as TCheckBox).Checked;
  cbColors.Invalidate;
end;

procedure TForm1.btnConnectClick(Sender: TObject);
begin
  AcConnect(rsDefClassName, False, True);
end;

procedure TForm1.btnAddLineClick(Sender: TObject);
var
  Line: OleVariant;
begin
  if not VarIsClear(Acad) then
  begin
    Line:= Acad.ActiveDocument.ModelSpace.AddLine(
                                          AcadPoint(Random(50), Random(50)),
                                          AcadPoint(Random(200), Random(200)));
    Line.Color:= cbColors.ItemIndex;
    Acad.Update;
  end;
end;

procedure TForm1.pbSamplePaint(Sender: TObject);
begin
  pbSample.Canvas.Brush.Color:= RGB(trbarR.Position, trbarG.Position, trbarB.Position);
  pbSample.Canvas.FillRect(pbSample.ClientRect);
end;

procedure TForm1.trbarRChange(Sender: TObject);
begin
  ColorTrackBarChange(Sender);
end;

procedure TForm1.ColorTrackBarChange(Sender: TObject);
var
  lbl: TLabel;
begin
  pbSample.Invalidate;
  with Sender as TTrackBar do
  begin
    lbl:= nil;
    if Name = 'trbarR' then lbl:= lblR else
    if Name = 'trbarG' then lbl:= lblG else
    if Name = 'trbarB' then lbl:= lblB;
    if lbl <> nil then
      lbl.Caption:= Format('%d', [Position]);
  end;
end;

procedure TForm1.btnAddTrueColorLineClick(Sender: TObject);
var
  AcCmColor, Line: OleVariant;
begin
  if not VarIsClear(Acad) then
  begin
    Line:= Acad.ActiveDocument.ModelSpace.AddLine(
                                          AcadPoint(Random(50), Random(50)),
                                          AcadPoint(Random(200), Random(200)));
    if chbUseGetIntfObj.Checked then
      AcCmColor:= Acad.GetInterfaceObject('AutoCAD.AcCmColor.16')
    else
      AcCmColor:= Line.TrueColor;
    AcCmColor.SetRGB(trbarR.Position, trbarG.Position, trbarB.Position);
    Line.TrueColor:= AcCmColor;
    Acad.Update;
  end;
end;

end.
